@@ -34,7 +34,7 @@ class Write(Read):
     def insert_documents(
         self,
         documents: list,
-        max_workers: Optional[int] = 2,
+        max_workers: Optional[int] = None,
         media_workers: Optional[int] = None,
         show_progress_bar: bool = True,
         chunksize: Optional[int] = None,
@@ -542,7 +542,6 @@ class Write(Read):
             func=bulk_func,
             func_args=bulk_func_args,
             func_kwargs=bulk_func_kwargs,
-            multithreaded_update=multithreaded_update,
             pull_chunksize=chunksize,
             transform_chunksize=transform_chunksize,
             push_chunksize=chunksize,
@@ -605,7 +604,6 @@ class Write(Read):
             return cat_vector_documents
 
         self.bulk_apply(
-            self.dataset_id,
             bulk_func=cat_fields,
             bulk_func_kwargs=dict(field_name=vector_name),
         )
