@@ -87,10 +87,11 @@ class BatchClusterTransform(ClusterTransform, ClusterAlias):
         """
         # run fit predict on documetns
         if hasattr(self.model, "predict_documents"):
-            return self.model.predict_documents(
+            cluster_labels = self.model.predict_documents(
                 documents=documents,
                 vector_fields=self.vector_fields,
             )
+            return self.format_cluster_labels(cluster_labels)
         elif hasattr(self.model, "predict"):
             if len(self.vector_fields) == 1:
                 vectors = self.get_field_across_documents(
@@ -100,7 +101,10 @@ class BatchClusterTransform(ClusterTransform, ClusterAlias):
                 cluster_labels = self.model.predict(vectors)
 
                 return self.format_cluster_labels(cluster_labels)
-        raise AttributeError("Model is missing a `fit_predict` method.")
+
+        raise AttributeError(
+            "Model is missing a `predict` or `predict_documents` method."
+        )
 
     def _get_model(self, model: Any, model_kwargs: dict) -> Any:
         if isinstance(model, str):
