@@ -1,10 +1,10 @@
 from typing import Optional
 
-from relevanceai.operations_new.apibase import OperationAPIBase
-from relevanceai.operations_new.dr.base import DimReductionBase
+from relevanceai.operations_new.ops_base import OperationAPIBase
+from relevanceai.operations_new.dr.transform import DimReductionTransform
 
 
-class DimReductionOps(DimReductionBase, OperationAPIBase):
+class DimReductionOps(DimReductionTransform, OperationAPIBase):
     """
     API related Functionality for Operation
     """
@@ -15,6 +15,7 @@ class DimReductionOps(DimReductionBase, OperationAPIBase):
         model,
         n_components: int,
         model_kwargs: Optional[dict] = None,
+        output_field: str = None,
         **kwargs
     ):
         if model_kwargs is None:
@@ -25,6 +26,7 @@ class DimReductionOps(DimReductionBase, OperationAPIBase):
             n_components=n_components,
             model_kwargs=model_kwargs,
             alias=alias,
+            output_field=output_field,
             **kwargs,
         )
         self.model = self._get_model(
