@@ -1,3 +1,4 @@
+import traceback
 from typing import List, Dict, Any, Optional
 
 from relevanceai.dataset import Dataset
@@ -76,11 +77,50 @@ class OperationManager:
         for h in self.post_hooks:
             h()
 
+        self._store_default_parent_child()
+
         self.operation.store_operation_metadata(
             dataset=self.dataset,
             values=self.metadata,
         )
 
+    def _store_default_parent_child(self):
+        """We temporarily store the default parent child relationship
+        where possible
+        """
+        try:
+            if hasattr(self.operation, "output_fields"):
+                if hasattr(self.operation, "select_fields"):
+                    for i, field in enumerate(self.operation.select_fields):
+                        self.dataset.update_field_children(
+                            field=field,
+                            field_children=[self.operation.output_fields[i]],
+                            category=self.operation.name,  # Should this be the workflow ID
+                            metadata={},
+                        )
+                elif hasattr(self.operation, "fields"):
+                    for i, field in enumerate(self.operation.fields):
+                        self.dataset.update_field_children(
+                            field=field,
+                            field_children=[self.operation.output_fields[i]],
+                            category=self.operation.name,  # Should this be the workflow ID
+                            metadata={},
+                        )
+                elif hasattr(self.operation, "text_fields"):
+                    for i, field in enumerate(self.operation.text_fields):
+                        self.dataset.update_field_children(
+                            field=field,
+                            field_children=[self.operation.output_fields[i]],
+                            category=self.operation.name,  # Should this be the workflow ID
+                            metadata={},
+                        )
+        except Exception as e:
+            # TODO: rigorously test this with different operations
+            # reason: `output_fields` are all calculated differently in
+            # different spots for different operations
+
+            traceback.print_exc()
+
     @staticmethod
     def clean(
         before_docs: List[Dict[str, Any]],
