@@ -11,7 +11,8 @@ from tests.globals.constants import generate_dataset_id
 def sample_dataset_id(test_client: Client, simple_documents: List[Dict]):
     test_dataset_id = generate_dataset_id()
 
-    test_client._insert_documents(test_dataset_id, simple_documents)
+    test_dataset = test_client.Dataset(test_dataset_id)
+    test_dataset.insert_documents(simple_documents)
 
     yield test_dataset_id
 
